@@ -3,20 +3,33 @@ package  Hessian::Deserializer::Date;
 use Moose::Role;
 
 use Switch;
+use YAML;
+use DateTime;
+use DateTime::Format::Epoch;
+use integer;
 
 sub read_date_handle_chunk  {    #{{{
     my ( $self, $first_bit,) = @_;
     my $input_handle = $self->input_handle();
+    my $formatter = DateTime::Format::Epoch->new(
+        unit  => 'milliseconds',
+        type  => 'bigint',
+        epoch => DateTime->new(
+            year      => 1970,
+            month     => 1,
+            day       => 1,
+            time_zone => 'UTC'
+        )
+    );
     my ( $date, $data );
     switch ($first_bit) {
-        case /\x4a/ {
+        case /\x64/ {
             $data = $self->read_long_handle_chunk( 'L');
         }
-        case /[\x4b\x64]/ {
-            $data = $self->read_integer_handle_chunk( 'I');
-        }
     }
-    return $data;
+    my $datetime = $formatter->parse_datetime($data);
+    $datetime->set_time_zone('UTC');
+    return $datetime;
 }    #}}}
 
 "one, but we're not the same";
